#-----------------------------------------------------------------------------
# Version: 1.0.0
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#-----------------------------------------------------------------------------
#
# CHANGELOG
# 1.0.0 - 16.03.2010
#		 * Initial Release.
#
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

module TT_ViewDirection
	
	@enabled = false
	@observer = nil
	@note = nil
	
	@debug = false
	
	### MENU & TOOLBARS ### --------------------------------------------------
	
	unless file_loaded?( File.basename(__FILE__) )
		view_menu = UI.menu('View')
		item = view_menu.add_item('View Direction')		{ self.toggle_viewdirection }
		view_menu.set_validation_proc(item)  {
			(@enabled) ? MF_CHECKED : MF_UNCHECKED
		}
	end
	
	
	### MAIN SCRIPT ### ------------------------------------------------------

	def self.toggle_viewdirection
		model = Sketchup.active_model
		@observer = VD_Observer.new if @observer.nil?
		if @enabled
			self.stop
		else
			self.observe
		end
	end
	
	
	def self.enabled?
		return @enabled
	end
	
	
	def self.set_note(text = nil)
		@note = text
	end
	
	
	def self.observe(model = Sketchup.active_model)
		self.debug("> Observe - #{model}")
		model.shadow_info.add_observer(@observer)
		model.active_view.add_observer(@observer)
		if @note.nil? || @note.deleted?
			self.debug('> Adding note...')
			@note = model.add_note('View Direction:', 0.01, 0.02)
			@note.set_attribute('TT_ViewDirection', 'Note', 1)
		end
		@enabled = true
		self.update
	end
	
	
	def self.stop(model = Sketchup.active_model)
		self.debug("> Stop - #{model}")
		model.shadow_info.remove_observer(@observer)
		model.active_view.remove_observer(@observer)
		@note.erase! unless @note.nil? || @note.deleted?
		@note = nil
		@enabled = false
	end
	
	
	def self.update
		model = Sketchup.active_model
		si = model.shadow_info
		
		# Deactivate if @note is missing
		if @note.nil? || @note.deleted?
			self.debug('> Deactivate')
			self.stop
			return
		end
		
		north = si['NorthAngle']
		t = Geom::Transformation.rotation(ORIGIN, Z_AXIS.reverse, north.degrees)
		north_vector = ORIGIN.vector_to( ORIGIN.offset(Y_AXIS, 100).transform!(t) )
		
		c = model.active_view.camera
		if c.direction.parallel?(Z_AXIS)
			angle_from_north = 0
			view_direction = ( c.direction.samedirection?(Z_AXIS) ) ? 'Up' : 'Down'
		else
			eye = c.eye
			target = c.target
			eye.z = 0
			target.z = 0
			view_vector = eye.vector_to(target)
		
			angle_from_north = full_angle_between(north_vector, view_vector).radians
			view_direction = '<error>'
			names = %w( North North-East East South-East South South-West West North-West North )
			step = 360.0 / (names.length-1)
			halfstep = step / 2
			for i in 0..names.length-1
				s1 = (i * step) - halfstep
				s2 = (i * step) + halfstep
				#view_direction = names[i] + " (#{s1} - #{s2}) - Step: #{step} - I: #{i}"
				if angle_from_north >= s1 && angle_from_north < s2
					view_direction = names[i]
					break
				end
			end
		end
		
		str = "View Direction: #{view_direction}\n"
		str << "Angle From North: #{sprintf('%.2f', angle_from_north)}\n\n"
		str << "North Angle: #{si['NorthAngle']}\n"
		str << "City: #{si['City']}\n"
		str << "Country: #{si['Country']}\n\n"
		str << "Latitude: #{si['Latitude']}\n"
		str << "Longitude: #{si['Longitude']}\n\n"
		str << "Timezone: #{si['TZOffset']}\n"
		@note.text = str
	end
	
	
	def self.full_angle_between(vector1, vector2)		
		cross_vector = vector1 * vector2
		direction = (vector1 * vector2) % Z_AXIS
		angle = vector1.angle_between(vector2)
		angle = 360.degrees - angle if direction > 0.0
		return angle
	end
	
	
	class VD_Observer
		
		# ViewObserver
		def onViewChanged(view)
			TT_ViewDirection.update
		end
		
		# ShadowInfoObserver
		def onShadowInfoChanged(shadow_info, type)
			TT_ViewDirection.update
			#TT_ViewDirection.debug("Shadow Info Changed: #{type}")
		end
		
		# AppObserver
		def onNewModel(model)
			TT_ViewDirection.debug("> VD_Observer.onNewModel #{model} - enabled: #{TT_ViewDirection.enabled?}")
			TT_ViewDirection.observe(model) if TT_ViewDirection.enabled?
		end
		
		# AppObserver
		def onOpenModel(model)
			TT_ViewDirection.debug("> VD_Observer.onOpenModel #{model} - enabled: #{TT_ViewDirection.enabled?}")
			if TT_ViewDirection.enabled?
				TT_ViewDirection.debug('> Looking for note...')
				model.entities.each { |e|
					if e.is_a?(Sketchup::Text) && e.get_attribute('TT_ViewDirection', 'Note') == 1
						TT_ViewDirection.debug('> Found note!')
						TT_ViewDirection.set_note(e)
						break
					end
				}
				TT_ViewDirection.observe(model)
			end
		end
		
	end
	
	
	def self.debug(object)
		puts object if @debug
	end
	
	
	unless file_loaded?( File.basename(__FILE__) )
		Sketchup.add_observer(@observer = VD_Observer.new)
	end

	
	### HELPER METHODS ### ---------------------------------------------------
	
	def self.start_operation(name)
		model = Sketchup.active_model
		if Sketchup.version.split('.')[0].to_i >= 7
			model.start_operation(name, true)
		else
			model.start_operation(name)
		end
	end
	
end # module

#-----------------------------------------------------------------------------
file_loaded( File.basename(__FILE__) )
#-----------------------------------------------------------------------------